@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2008-2011 Quinox <quinox@users.sf.net>
 # COPYRIGHT (C) 2008 Gallows <g4ll0ws@gmail.com>
@@ -21,17 +20,18 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import gtk
-import gobject
-import os
-import sys
-import re
-import thread
-import threading
 import copy
+import os
+import re
 import sys
-from pynicotine.utils import executeCommand
+from gettext import gettext as _
+
+from gi.repository import Gdk
+from gi.repository import GObject as gobject
+from gi.repository import Gtk as gtk
+
 from pynicotine.logfacility import log
+from pynicotine.utils import executeCommand
 
 
 class NowPlaying:
@@ -136,8 +136,6 @@ class NowPlaying:
             self.NP_audacious.set_active(1)
         elif player == "mpd":
             self.NP_mpd.set_active(1)
-        elif player == "banshee":
-            self.NP_banshee.set_active(1)
         elif player == "exaile":
             self.NP_exaile.set_active(1)
         elif player == "lastfm":
@@ -159,10 +157,7 @@ class NowPlaying:
         isset = False
 
         if self.NP_mpd.get_active():
-            self.player_replacers = ["$n", "$t", "$a", "$b",  "$f", "$k"]
-            isset = True
-        elif self.NP_banshee.get_active():
-            self.player_replacers = ["$n", "$t", "$l", "$a", "$b", "$k", "$y", "$r", "$f"]
+            self.player_replacers = ["$n", "$t", "$a", "$b", "$f", "$k"]
             isset = True
         elif self.NP_amarok.get_active():
             self.player_replacers = ["$n", "$t", "$l", "$a", "$b", "$c", "$k", "$y", "$r", "$f"]
@@ -237,7 +232,7 @@ class NowPlaying:
     def OnKeyPress(self, widget, event):
 
         # Close the window when escape is pressed
-        if event.keyval == gtk.keysyms.Escape:
+        if event.keyval == Gdk.KEY_Escape:
             self.quit(None)
 
     def quit(self, widget, s=None):
@@ -245,7 +240,7 @@ class NowPlaying:
         # Save new defined formats in npformatlist before exiting
         config = self.frame.np.config.sections
 
-        text = self.NPFormat.child.get_text()
+        text = self.NPFormat.get_child().get_text()
 
         if text is not None and not text.isspace() and text != "":
             if text not in config["players"]["npformatlist"] and text not in self.defaultlist:
@@ -262,12 +257,7 @@ class NowPlaying:
 
     def DisplayNowPlaying(self, widget, test=0, callback=None):
 
-        if self.NP_mpris.get_active():
-            # dbus (no threads, please)
-            self.GetNP(None, test, callback)
-        else:
-            # thread (command execution)
-            thread.start_new_thread(self.GetNP, (None, test, callback))
+        self.GetNP(None, test, callback)
 
     def GetNP(self, widget, test=None, callback=None):
 
@@ -283,8 +273,6 @@ class NowPlaying:
                 result = self.audacious()
             elif self.NP_mpd.get_active():
                 result = self.mpd()
-            elif self.NP_banshee.get_active():
-                result = self.banshee()
             elif self.NP_exaile.get_active():
                 result = self.exaile()
             elif self.NP_lastfm.get_active():
@@ -310,13 +298,13 @@ class NowPlaying:
         # - this is a failsafe.
         oldtitle = copy.copy(self.title)
         self.title_clear()
-        for key, value in oldtitle.iteritems():
+        for key, value in oldtitle.items():
             try:
-                self.title[key] = unicode(value, "UTF-8", "replace")
+                self.title[key] = str(value, "UTF-8", "replace")
             except TypeError:
                 self.title[key] = value  # already unicode
 
-        title = self.NPFormat.child.get_text()
+        title = self.NPFormat.get_child().get_text()
         title = title.replace("%", "%%")  # Escaping user supplied % symbols
         title = title.replace("$t", "%(title)s")
         title = title.replace("$a", "%(artist)s")
@@ -352,8 +340,6 @@ class NowPlaying:
             player = "audacious"
         elif self.NP_mpd.get_active():
             player = "mpd"
-        elif self.NP_banshee.get_active():
-            player = "banshee"
         elif self.NP_exaile.get_active():
             player = "exaile"
         elif self.NP_lastfm.get_active():
@@ -369,14 +355,14 @@ class NowPlaying:
 
         self.frame.np.config.sections["players"]["npplayer"] = player
         self.frame.np.config.sections["players"]["npothercommand"] = self.NPCommand.get_text()
-        self.frame.np.config.sections["players"]["npformat"] = self.NPFormat.child.get_text()
+        self.frame.np.config.sections["players"]["npformat"] = self.NPFormat.get_child().get_text()
         self.frame.np.config.writeConfiguration()
 
         self.quit(None)
 
     def mpd(self):
 
-        format = self.NPFormat.child.get_text()
+        format = self.NPFormat.get_child().get_text()
 
         if "$a" in format:
             output = self.mpd_command("%artist%")
@@ -407,83 +393,13 @@ class NowPlaying:
 
     def mpd_command(self, command):
 
-        output = executeCommand("mpc --format $", command, returnoutput=True).split('\n')[0]
+        output = executeCommand("mpc --format $", command, returnoutput=True).decode().split('\n')[0]
 
         if output == '' or output.startswith("MPD_HOST") or output.startswith("volume: "):
             return None
 
         return output
 
-    def banshee(self):
-        """ Function to get banshee currently playing song """
-
-        from urllib import unquote
-
-        commandlist = [
-            "--query-title",
-            "--query-artist",
-            "--query-album",
-            "--query-track-count",
-            "--query-duration",
-            "--query-year",
-            "--query-bit-rate",
-            "--query-uri"
-        ]
-
-        output = self.banshee_command(commandlist)
-
-        matches = {}
-        [matches.__setitem__(i[0].split(':')[0], filter(len, i[1:])[0]) for i in re.findall(r"(?m)^(title: (?P<title>.*?)|artist: (?P<artist>.*?)|album: (?P<album>.*?)|track-number: (?P<track>.*?)|duration: (?P<length>.*?)|year: (?P<year>.*?)|bit-rate: (?P<bitrate>.*?)|uri: (?P<filename>.*?))$", output)]
-
-        if matches:
-
-            self.title["nowplaying"] = "%(artist)s - %(title)s" % matches
-
-            for key, value in matches.iteritems():
-
-                if key == "duration":
-                    value = value.replace(',', '.')
-                    self.title["length"] = self.get_length_time(float(value))
-                elif key == "bit-rate":
-                    self.title["bitrate"] = value
-                elif key == "track-number":
-                    self.title["track"] = value
-                elif key == "uri":
-                    value = unquote(value)
-                    self.title["filename"] = value.split('://')[1]
-                else:
-                    self.title[key] = value
-
-            return True
-        else:
-            return False
-
-    def banshee_command(self, commands):
-        """ Wrapper that calls banshee commandline """
-
-        return executeCommand(" ".join(["banshee"] + commands), returnoutput=True)
-
-    def exaile(self):
-        """ Function to get exaile currently playing song """
-
-        # At this time exail doesn't support mpris2: it will com with exaile 4
-        # So we use the command line to query it
-        output = executeCommand('exaile --get-album --get-artist --get-length --get-title', returnoutput=True)
-        output = output.split('\n')
-
-        self.title["title"] = output[0]
-        self.title["artist"] = output[1]
-        self.title["album"] = output[2]
-        self.title["length"] = self.get_length_time(float(output[3]))
-
-        if self.title['artist'] != "":
-            self.title['nowplaying'] += self.title['artist']
-
-        if self.title['title'] != "":
-            self.title['nowplaying'] += " - " + self.title['title']
-
-        return True
-
     def amarok(self):
         """ Function to get amarok currently playing song """
 
@@ -499,7 +415,7 @@ class NowPlaying:
         player = self.bus.get_object('org.mpris.amarok', '/Player')
         md = player.GetMetadata()
 
-        for key, value in md.iteritems():
+        for key, value in md.items():
 
             if key == 'mtime':
                 # Convert seconds to minutes:seconds
@@ -520,7 +436,7 @@ class NowPlaying:
     def audacious(self):
         """ Function to get audacious currently playing song """
 
-        slist = self.NPFormat.child.get_text()
+        slist = self.NPFormat.get_child().get_text()
         output = ""
         self.audacious_running = True
 
@@ -571,7 +487,7 @@ class NowPlaying:
                 self.title["bitrate"] = output
 
         if "$f" in slist:
-            path = self.audacious_command('current-song-filename')
+            path = self.audacious_command('current-song-filename')  # noqa: F841
 
         if not self.audacious_running:
             log.addwarning(_("ERROR: audacious: audtool didn't detect a running Audacious session."))
@@ -583,9 +499,9 @@ class NowPlaying:
         """ Wrapper that calls audacious commandline audtool and parse the output """
 
         try:
-            output = executeCommand("audtool %s %s" % (command, subcommand), returnoutput=True).split('\n')[0]
+            output = executeCommand("audtool %s %s" % (command, subcommand), returnoutput=True).decode().split('\n')[0]
         except RuntimeError:
-            output = executeCommand("audtool2 %s %s" % (command, subcommand), returnoutput=True).split('\n')[0]
+            output = executeCommand("audtool2 %s %s" % (command, subcommand), returnoutput=True).decode().split('\n')[0]
 
         if output.startswith('audtool'):
             output = None
@@ -608,10 +524,10 @@ class NowPlaying:
 
         player = self.NPCommand.get_text()
 
-        dbus_mpris_service = u'org.mpris.MediaPlayer2.'
-        dbus_mpris_player_service = u'org.mpris.MediaPlayer2.Player'
-        dbus_mpris_path = u'/org/mpris/MediaPlayer2'
-        dbus_property = u'org.freedesktop.DBus.Properties'
+        dbus_mpris_service = 'org.mpris.MediaPlayer2.'
+        dbus_mpris_player_service = 'org.mpris.MediaPlayer2.Player'
+        dbus_mpris_path = '/org/mpris/MediaPlayer2'
+        dbus_property = 'org.freedesktop.DBus.Properties'
 
         if not player:
 
@@ -636,7 +552,7 @@ class NowPlaying:
             player_obj = self.bus.get_object(dbus_mpris_service + player, dbus_mpris_path)
             player_property_obj = dbus.Interface(player_obj, dbus_interface=dbus_property)
             metadata = player_property_obj.Get(dbus_mpris_player_service, "Metadata")
-        except Exception, exception:
+        except Exception as exception:
             log.addwarning(_("ERROR: MPRIS: Something went wrong while querying %(player)s: %(exception)s") % {'player': player, 'exception': exception})
             return None
 
@@ -650,16 +566,16 @@ class NowPlaying:
                 self.title[dest] = '?'
 
         mapping = [
-                ('xesam:title', 'title'),
-                ('xesam:album', 'album'),
-                ('xesam:comment', 'comment'),
-                ('xesam:audioBitrate', 'bitrate'),
-                ('xesak:trackNumber', 'track')
-            ]
+            ('xesam:title', 'title'),
+            ('xesam:album', 'album'),
+            ('xesam:comment', 'comment'),
+            ('xesam:audioBitrate', 'bitrate'),
+            ('xesak:trackNumber', 'track')
+        ]
 
         for (source, dest) in mapping:
             try:
-                self.title[dest] = unicode(metadata[source])
+                self.title[dest] = str(metadata[source])
             except KeyError:
                 self.title[dest] = '?'
 
@@ -701,9 +617,9 @@ class NowPlaying:
         for wnd_id in wnd_ids:
             wnd_txt = GetWindowText(FindWindow(wnd_id, None))
             if wnd_txt:
-                m = re.match("(.*)\s+\[foobar.*", wnd_txt)
+                m = re.match(r"(.*)\\s+\[foobar.*", wnd_txt)
                 if m:
-                    metadata = m.groups(0)[0].strip()
+                    metadata = m.groups()[0].strip()
 
         if metadata:
             self.title["nowplaying"] = "now playing: " + metadata.decode('mbcs')
@@ -716,7 +632,7 @@ class NowPlaying:
 
         if length != '' and length is not None:
 
-            minutes = int(length)/60
+            minutes = int(length) / 60
             seconds = str(int(length) - (60 * minutes))
 
             if len(seconds) < 2:
@@ -731,18 +647,18 @@ class NowPlaying:
     def lastfm(self):
         """ Function to get the last song played via lastfm api """
 
-        import httplib
+        import http.client
         import json
 
         try:
-            conn = httplib.HTTPConnection("ws.audioscrobbler.com")
+            conn = http.client.HTTPConnection("ws.audioscrobbler.com")
         except Exception as error:
             log.addwarning(_("ERROR: lastfm: Could not connect to audioscrobbler: %(error)s") % {"error": error})
             return None
 
         try:
             (user, apikey) = self.NPCommand.get_text().split(';')
-        except ValueError as error:
+        except ValueError as error:  # noqa: F841
             log.addwarning(_("ERROR: lastfm: Please provide both your lastfm username and API key"))
             return None
 
@@ -773,7 +689,7 @@ class NowPlaying:
             output = executeCommand(othercommand, returnoutput=True)
             self.title["nowplaying"] = output
             return True
-        except Exception, error:
+        except Exception as error:
             log.addwarning(_("ERROR: Executing '%(command)s' failed: %(error)s") % {"command": othercommand, "error": error})
             return None
 
@@ -792,7 +708,7 @@ class NowPlaying:
         # Now we need to connect to xmms2d
         try:
             xmms.connect(os.getenv("XMMS_PATH"))
-        except IOError as detail:
+        except IOError as error:
             log.addwarning(_("ERROR: xmms2: connecting failed: %(error)s") % {"error": error})
             return None
 
