@@ -1,5 +1,4 @@
-# -*- coding: utf-8 -*-
-#
+# COPYRIGHT (C) 2020 Nicotine+ Team
 # COPYRIGHT (C) 2016-2017 Michael Labouebe <gfarmerfr@free.fr>
 # COPYRIGHT (C) 2016 Mutnick <muhing@yahoo.com>
 # COPYRIGHT (C) 2009-2011 Quinox <quinox@users.sf.net>
@@ -21,24 +20,46 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-# System imports
-import dircache
-import gobject
+import os
 import string
 import sys
-import os
 import time
-import struct
+from gettext import gettext as _
+
+from gi.repository import GLib
+
+from pynicotine import metadata_mutagen as metadata
+from pynicotine import slskmessages
+from pynicotine.logfacility import log
+from pynicotine.slskmessages import NetworkIntType
+from pynicotine.slskmessages import NetworkLongLongType
+from pynicotine.utils import displayTraceback
+from pynicotine.utils import GetUserDirectories
 
-import slskmessages
-from slskmessages import NetworkIntType, NetworkLongLongType
-from logfacility import log
-from utils import displayTraceback
-import metadata_mutagen as metadata
 
 win32 = sys.platform.startswith("win")
 
 
+class dircache:
+    """Adapted from https://raw.githubusercontent.com/python/cpython/2.7/Lib/dircache.py"""
+
+    cache = {}
+
+    @classmethod
+    def listdir(cls, path):
+        try:
+            cached_mtime, dir_list = cls.cache[path]
+            del cls.cache[path]
+        except KeyError:
+            cached_mtime, dir_list = -1, []
+        mtime = os.stat(path).st_mtime
+        if mtime != cached_mtime:
+            dir_list = os.listdir(path)
+            dir_list.sort()
+        cls.cache[path] = mtime, dir_list
+        return dir_list
+
+
 class Shares:
 
     def __init__(self, np):
@@ -51,9 +72,9 @@ class Shares:
         self.CompressShares("buddy")
         self.requestedShares = {}
         self.newbuddyshares = self.newnormalshares = False
-        self.translatepunctuation = string.maketrans(
+        self.translatepunctuation = str.maketrans(
             string.punctuation,
-            string.join([' ' for i in string.punctuation], '')
+            ''.join([' ' for i in string.punctuation])
         )
 
     def real2virtual(self, path):
@@ -88,7 +109,7 @@ class Shares:
 
     def logMessage(self, message, debugLevel=0):
         if self.LogMessage is not None:
-            gobject.idle_add(self.LogMessage, message, debugLevel)
+            GLib.idle_add(self.LogMessage, message, debugLevel)
 
     def sendNumSharedFoldersFiles(self):
         """
@@ -103,7 +124,7 @@ class Shares:
             shared_db = "sharedfiles"
 
         sharedfolders = len(conf["transfers"][shared_db])
-        sharedfiles = sum([len(x) for x in conf["transfers"][shared_db].values()])
+        sharedfiles = sum([len(x) for x in list(conf["transfers"][shared_db].values())])
         self.queue.put(slskmessages.SharedFoldersFiles(sharedfolders, sharedfiles))
 
     def RebuildShares(self, msg):
@@ -129,8 +150,11 @@ class Shares:
                 [files, streams, wordindex, fileindex, mtimes],
                 "normal"
             )
-        except Exception, ex:
-            log.addwarning(_("Failed to rebuild share, serious error occurred. If this problem persists delete ~/.nicotine/*.db and try again. If that doesn't help please file a bug report with the stack trace included (see terminal output after this message). Technical details: %s") % ex)
+        except Exception as ex:
+            config_dir, data_dir = GetUserDirectories()
+            log.addwarning(
+                _("Failed to rebuild share, serious error occurred. If this problem persists delete %s/*.db and try again. If that doesn't help please file a bug report with the stack trace included (see terminal output after this message). Technical details: %s") % (data_dir, ex)
+            )
             raise
 
     def RebuildBuddyShares(self, msg):
@@ -165,7 +189,7 @@ class Shares:
 
         if streams is None:
             message = _("ERROR: No %(type)s shares database available") % {"type": sharestype}
-            print message
+            print(message)
             self.logMessage(message, None)
             return
 
@@ -227,8 +251,8 @@ class Shares:
             return
 
         self.logMessage(_("%(user)s is making a BrowseShares request") % {
-                'user': user
-            }, 1)
+            'user': user
+        }, 1)
 
         addr = msg.conn.addr[0]
         checkuser, reason = self.np.CheckUser(user, addr)
@@ -278,7 +302,7 @@ class Shares:
         elif checkuser == 2:
             shares = self.config.sections["transfers"]["bsharedfiles"]
         else:
-            response = self.queue.put(slskmessages.TransferResponse(msg.conn.conn, 0, reason=reason, req=0))
+            response = self.queue.put(slskmessages.TransferResponse(msg.conn.conn, 0, reason=reason, req=0))  # noqa: F841
             shares = {}
 
         if checkuser:
@@ -296,10 +320,6 @@ class Shares:
 
         self.logMessage("%s %s" % (msg.__class__, vars(msg)), 4)
 
-    def processExactSearchRequest(self, searchterm, user, searchid,  direct=0, checksum=None):
-        print searchterm, user, searchid, checksum
-        pass
-
     def processSearchRequest(self, searchterm, user, searchid, direct=0):
 
         if not self.config.sections["searches"]["search_results"]:
@@ -309,6 +329,10 @@ class Shares:
         if searchterm is None:
             return
 
+        if user != self.config.sections["server"]["login"]:
+            # We shouldn't send a search response if we initiated the search request
+            return
+
         checkuser, reason = self.np.CheckUser(user, None)
         if not checkuser:
             return
@@ -374,14 +398,14 @@ class Shares:
                     self.logMessage(
                         _("User %(user)s is directly searching for %(query)s, returning %(num)i results") % {
                             'user': user,
-                            'query': self.np.decode(searchterm),
+                            'query': searchterm,
                             'num': len(results)
                         }, 2)
                 else:
                     self.logMessage(
                         _("User %(user)s is searching for %(query)s, returning %(num)i results") % {
                             'user': user,
-                            'query': self.np.decode(searchterm),
+                            'query': searchterm,
                             'num': len(results)
                         }, 2)
 
@@ -395,9 +419,9 @@ class Shares:
         # returns dict in format:  { Directory : mtime, ... }
         shared_directories = [x[1] for x in shared]
 
-        gobject.idle_add(progress.set_text, _("Checking for changes"))
-        gobject.idle_add(progress.show)
-        gobject.idle_add(progress.set_fraction, 0)
+        GLib.idle_add(progress.set_text, _("Checking for changes"))
+        GLib.idle_add(progress.show)
+        GLib.idle_add(progress.set_fraction, 0)
 
         self.logMessage(_("%(num)s directories found before rescan/rebuild") % {"num": len(oldmtimes)})
 
@@ -408,7 +432,7 @@ class Shares:
 
         self.logMessage(_("%(num)s directories found after rescan/rebuild") % {"num": len(newmtimes)})
 
-        gobject.idle_add(progress.set_text, _("Scanning %s") % name)
+        GLib.idle_add(progress.set_text, _("Scanning %s") % name)
 
         # Get list of files
         # returns dict in format { Directory : { File : metadata, ... }, ... }
@@ -419,7 +443,7 @@ class Shares:
 
         # Pack shares data
         # returns dict in format { Directory : hex string of files+metadata, ... }
-        gobject.idle_add(progress.set_text, _("Building DataBase"))
+        GLib.idle_add(progress.set_text, _("Building DataBase"))
 
         if win32:
             newsharedfilesstreams = self.getFilesStreamsUnicode(newmtimes, oldmtimes, sharedfilesstreams, newsharedfiles, rebuild, yieldfunction)
@@ -430,13 +454,13 @@ class Shares:
         # newwordindex is a dict in format {word: [num, num, ..], ... } with num matching
         # keys in newfileindex
         # newfileindex is a dict in format { num: (path, size, (bitrate, vbr), length), ... }
-        gobject.idle_add(progress.set_text, _("Building Index"))
+        GLib.idle_add(progress.set_text, _("Building Index"))
 
-        gobject.idle_add(progress.set_fraction, 0.0)
+        GLib.idle_add(progress.set_fraction, 0.0)
 
         newwordindex, newfileindex = self.getFilesIndex(newmtimes, oldmtimes, shared_directories, newsharedfiles, yieldfunction, progress)
 
-        gobject.idle_add(progress.set_fraction, 1.0)
+        GLib.idle_add(progress.set_fraction, 1.0)
 
         return newsharedfiles, newsharedfilesstreams, newwordindex, newfileindex, newmtimes
 
@@ -455,9 +479,9 @@ class Shares:
             try:
                 contents = dircache.listdir(directory)
                 mtime = os.path.getmtime(directory)
-            except OSError, errtuple:
+            except OSError as errtuple:
                 message = _("Scanning Directory Error: %(error)s Path: %(path)s") % {'error': errtuple, 'path': directory}
-                print str(message)
+                print(str(message))
                 self.logMessage(message)
                 displayTraceback(sys.exc_info()[2])
                 continue
@@ -472,20 +496,20 @@ class Shares:
 
                 try:
                     isdir = os.path.isdir(path)
-                except OSError, errtuple:
+                except OSError as errtuple:
                     message = _("Scanning Error: %(error)s Path: %(path)s") % {'error': errtuple, 'path': path}
-                    print str(message)
+                    print(str(message))
                     self.logMessage(message)
                     continue
 
                 try:
                     mtime = os.path.getmtime(path)
-                except OSError, errtuple:
+                except OSError as errtuple:
                     islink = False
                     try:
                         islink = os.path.islink(path)
-                    except OSError, errtuple2:
-                        print errtuple2
+                    except OSError as errtuple2:
+                        print(errtuple2)
 
                     if islink:
                         message = _("Scanning Error: Broken link to directory: \"%(link)s\" from Path: \"%(path)s\". Repair or remove this link.") % {
@@ -498,7 +522,7 @@ class Shares:
                             'path': path
                         }
 
-                    print str(message)
+                    print(str(message))
                     self.logMessage(message)
                     continue
                 else:
@@ -527,9 +551,9 @@ class Shares:
             count += 1
 
             if progress:
-                percent = float(count)/len(mtimes)
+                percent = float(count) / len(mtimes)
                 if percent <= 1.0:
-                    gobject.idle_add(progress.set_fraction, percent)
+                    GLib.idle_add(progress.set_fraction, percent)
 
             if self.hiddenCheck({'dir': directory}):
                 continue
@@ -553,8 +577,8 @@ class Shares:
 
             try:
                 contents = os.listdir(directory)
-            except OSError, errtuple:
-                print str(errtuple)
+            except OSError as errtuple:
+                print(str(errtuple))
                 self.logMessage(str(errtuple))
                 continue
 
@@ -568,9 +592,9 @@ class Shares:
                 path = os.path.join(directory, filename)
                 try:
                     isfile = os.path.isfile(path)
-                except OSError, errtuple:
+                except OSError as errtuple:
                     message = _("Scanning Error: %(error)s Path: %(path)s") % {'error': errtuple, 'path': path}
-                    print str(message)
+                    print(str(message))
                     self.logMessage(message)
                     displayTraceback(sys.exc_info()[2])
                     continue
@@ -606,7 +630,7 @@ class Shares:
 
             return fileinfo
 
-        except Exception, errtuple:
+        except Exception as errtuple:
             message = _("Scanning File Error: %(error)s Path: %(path)s") % {'error': errtuple, 'path': pathname}
             self.logMessage(message)
             displayTraceback(sys.exc_info()[2])
@@ -615,9 +639,9 @@ class Shares:
     def getFilesStreams(self, mtimes, oldmtimes, oldstreams, newsharedfiles, rebuild=False, yieldcall=None):
 
         streams = {}
-        shared = self.config.sections["transfers"]["shared"]
+        shared = self.config.sections["transfers"]["shared"]  # noqa: F841
 
-        for directory in mtimes.keys():
+        for directory in list(mtimes.keys()):
 
             virtualdir = self.real2virtual(directory)
 
@@ -656,7 +680,7 @@ class Shares:
 
             directory = os.path.expanduser(directory.replace("//", "/"))
 
-            u_directory = u"%s" % directory
+            u_directory = "%s" % directory
             str_directory = str(directory)
 
             if self.hiddenCheck({'dir': directory}):
@@ -665,9 +689,9 @@ class Shares:
             try:
                 contents = dircache.listdir(u_directory)
                 mtime = os.path.getmtime(u_directory)
-            except OSError, errtuple:
+            except OSError as errtuple:
                 message = _("Scanning Directory Error: %(error)s Path: %(path)s") % {'error': errtuple, 'path': u_directory}
-                print str(message)
+                print(str(message))
                 self.logMessage(message)
                 displayTraceback(sys.exc_info()[2])
                 continue
@@ -681,25 +705,25 @@ class Shares:
                 path = os.path.join(directory, filename)
 
                 # force Unicode for reading from disk in win32
-                u_path = u"%s" % path
+                u_path = "%s" % path
                 s_path = str(path)
 
                 try:
                     isdir = os.path.isdir(u_path)
-                except OSError, errtuple:
+                except OSError as errtuple:
                     message = _("Scanning Error: %(error)s Path: %(path)s") % {'error': errtuple, 'path': u_path}
-                    print str(message)
+                    print(str(message))
                     self.logMessage(message)
                     continue
 
                 try:
                     mtime = os.path.getmtime(u_path)
-                except OSError, errtuple:
+                except OSError as errtuple:  # noqa: F841
                     try:
                         mtime = os.path.getmtime(s_path)
-                    except OSError, errtuple:
+                    except OSError as errtuple:
                         message = _("Scanning Error: %(error)s Path: %(path)s") % {'error': errtuple, 'path': u_path}
-                        print str(message)
+                        print(str(message))
                         self.logMessage(message)
                         continue
                 else:
@@ -728,13 +752,13 @@ class Shares:
             count += 1
 
             if progress:
-                percent = float(count)/len(mtimes)
+                percent = float(count) / len(mtimes)
                 if percent <= 1.0:
-                    gobject.idle_add(progress.set_fraction, percent)
+                    GLib.idle_add(progress.set_fraction, percent)
 
             # force Unicode for reading from disk
-            u_directory = u"%s" % directory
-            str_directory = str(directory)
+            u_directory = "%s" % directory
+            str_directory = str(directory)  # noqa: F841
 
             if self.hiddenCheck({'dir': directory}):
                 continue
@@ -762,8 +786,8 @@ class Shares:
 
             try:
                 contents = os.listdir(u_directory)
-            except OSError, errtuple:
-                print str(errtuple)
+            except OSError as errtuple:
+                print(str(errtuple))
                 self.logMessage(str(errtuple))
                 continue
 
@@ -776,15 +800,15 @@ class Shares:
 
                 path = os.path.join(directory, filename)
                 s_path = str(path)
-                ppath = unicode(path)
+                ppath = str(path)
 
                 s_filename = str(filename)
                 try:
                     # try to force Unicode for reading from disk
                     isfile = os.path.isfile(ppath)
-                except OSError, errtuple:
+                except OSError as errtuple:
                     message = _("Scanning Error: %(error)s Path: %(path)s") % {'error': errtuple, 'path': path}
-                    print str(message)
+                    print(str(message))
                     self.logMessage(message)
                     displayTraceback(sys.exc_info()[2])
                     continue
@@ -806,18 +830,18 @@ class Shares:
         try:
 
             if type(name) is str:
-                pathname_f = u"%s" % pathname
+                pathname_f = "%s" % pathname
             else:
                 pathname_f = pathname
 
             try:
                 size = os.path.getsize(pathname_f)
-            except:
+            except Exception:
                 size = os.path.getsize(pathname)
 
             try:
                 info = metadata.detect(pathname_f)
-            except:
+            except Exception:
                 info = metadata.detect(pathname)
 
             if info:
@@ -833,7 +857,7 @@ class Shares:
 
             return fileinfo
 
-        except Exception, errtuple:
+        except Exception as errtuple:
             message = _("Scanning File Error: %(error)s Path: %(path)s") % {'error': errtuple, 'path': pathname}
             self.logMessage(message)
             displayTraceback(sys.exc_info()[2])
@@ -842,15 +866,15 @@ class Shares:
     def getFilesStreamsUnicode(self, mtimes, oldmtimes, oldstreams, newsharedfiles, rebuild=False, yieldcall=None):
 
         streams = {}
-        shared = self.config.sections["transfers"]["shared"]
+        shared = self.config.sections["transfers"]["shared"]  # noqa: F841
 
-        for directory in mtimes.keys():
+        for directory in list(mtimes.keys()):
 
             virtualdir = self.real2virtual(directory)
 
             # force Unicode for reading from disk
-            u_directory = u"%s" % directory
-            str_directory = str(directory)
+            u_directory = "%s" % directory
+            str_directory = str(directory)  # noqa: F841
 
             if self.hiddenCheck({'dir': directory}):
                 continue
@@ -901,7 +925,7 @@ class Shares:
 
             try:
                 from win32file import GetFileAttributes
-            except ImportError as e:
+            except ImportError as e:  # noqa: F841
                 pass
             else:
 
@@ -911,7 +935,7 @@ class Shares:
                 else:
                     path = stuff['dir'].replace('\\', '\\\\')
 
-                attrs = GetFileAttributes(unicode(path))
+                attrs = GetFileAttributes(str(path))
 
                 # Set a mask to check the 2nd bit
                 # See https://msdn.microsoft.com/en-us/library/windows/desktop/gg258117(v=vs.85).aspx
@@ -940,10 +964,10 @@ class Shares:
 
         message = slskmessages.SlskMessage()
 
-        stream = chr(1) + message.packObject(fileinfo[0]) + message.packObject(NetworkLongLongType(fileinfo[1]))
+        stream = bytes([1]) + message.packObject(fileinfo[0]) + message.packObject(NetworkLongLongType(fileinfo[1]))
         if fileinfo[2] is not None:
             try:
-                msgbytes = ''
+                msgbytes = b''
                 msgbytes += message.packObject('mp3') + message.packObject(3)
                 msgbytes += (
                     message.packObject(0) +
@@ -954,13 +978,13 @@ class Shares:
                     message.packObject(NetworkIntType(fileinfo[2][1]))
                 )
                 stream += msgbytes
-            except:
+            except Exception:
                 log.addwarning(_("Found meta data that couldn't be encoded, possible corrupt file: '%(file)s' has a bitrate of %(bitrate)s kbs, a length of %(length)s seconds and a VBR of %(vbr)s" % {
-                        'file':    fileinfo[0],
-                        'bitrate': fileinfo[2][0],
-                        'length':  fileinfo[3],
-                        'vbr':     fileinfo[2][1]
-                    }))
+                    'file': fileinfo[0],
+                    'bitrate': fileinfo[2][0],
+                    'length': fileinfo[3],
+                    'vbr': fileinfo[2][1]
+                }))
                 stream += message.packObject('') + message.packObject(0)
         else:
             stream += message.packObject('') + message.packObject(0)
@@ -975,14 +999,14 @@ class Shares:
         index = 0
         count = 0
 
-        for directory in mtimes.keys():
+        for directory in list(mtimes.keys()):
 
             virtualdir = self.real2virtual(directory)
 
             if progress:
-                percent = float(count)/len(mtimes)
+                percent = float(count) / len(mtimes)
                 if percent <= 1.0:
-                    gobject.idle_add(progress.set_fraction, percent)
+                    GLib.idle_add(progress.set_fraction, percent)
 
             count += 1
 
@@ -993,7 +1017,7 @@ class Shares:
                 indexes = self.getIndexWords(virtualdir, j[0], shareddirs)
                 for k in indexes:
                     wordindex.setdefault(k, []).append(index)
-                fileindex[str(index)] = ((virtualdir + '\\' + j[0]), )+j[1:]
+                fileindex[str(index)] = ((virtualdir + '\\' + j[0]), ) + j[1:]
                 index += 1
 
             if yieldcall is not None:
@@ -1008,15 +1032,15 @@ class Shares:
             if os.path.commonprefix([dir, i]) == i:
                 dir = dir[len(i):]
 
-        words = string.split(
-            string.lower(string.translate(dir + ' ' + file, string.maketrans(string.punctuation, string.join([' ' for i in string.punctuation], ''))))
-        )
+        words = (dir + ' ' + file).translate(
+            str.maketrans(string.punctuation, ''.join([' ' for i in string.punctuation]))
+        ).lower().split()
 
         # remove duplicates
         d = {}
         for x in words:
             d[x] = x
-        return d.values()
+        return list(d.values())
 
     def addToShared(self, name):
         """ Add a file to the normal shares database """
@@ -1029,22 +1053,25 @@ class Shares:
         sharedstreams = config["transfers"]["sharedfilesstreams"]
         wordindex = config["transfers"]["wordindex"]
         fileindex = config["transfers"]["fileindex"]
-        shareddirs = config["transfers"]["shared"] + [config["transfers"]["downloaddir"]]
+
+        shareddirs = [path for _name, path in config["transfers"]["shared"]]
+        shareddirs.append(config["transfers"]["downloaddir"])
+
         sharedmtimes = config["transfers"]["sharedmtimes"]
 
         dir = str(os.path.expanduser(os.path.dirname(name)))
+        vdir = self.real2virtual(dir)
         file = str(os.path.basename(name))
-        size = os.path.getsize(name)
 
-        shared[dir] = shared.get(dir, [])
+        shared[vdir] = shared.get(vdir, [])
 
-        if file not in [i[0] for i in shared[dir]]:
+        if file not in [i[0] for i in shared[vdir]]:
             fileinfo = self.getFileInfo(file, name)
-            shared[dir] = shared[dir] + [fileinfo]
-            sharedstreams[dir] = self.getDirStream(shared[dir])
-            words = self.getIndexWords(dir, file, shareddirs)
-            self.addToIndex(wordindex, fileindex, words, dir, fileinfo)
-            sharedmtimes[dir] = os.path.getmtime(dir)
+            shared[vdir] = shared[vdir] + [fileinfo]
+            sharedstreams[vdir] = self.getDirStream(shared[vdir])
+            words = self.getIndexWords(vdir, file, shareddirs)
+            self.addToIndex(wordindex, fileindex, words, vdir, fileinfo)
+            sharedmtimes[vdir] = os.path.getmtime(dir)
             self.newnormalshares = True
 
         if config["transfers"]["enablebuddyshares"]:
@@ -1063,29 +1090,33 @@ class Shares:
         bsharedstreams = config["transfers"]["bsharedfilesstreams"]
         bwordindex = config["transfers"]["bwordindex"]
         bfileindex = config["transfers"]["bfileindex"]
-        bshareddirs = config["transfers"]["buddyshared"] + config["transfers"]["shared"] + [config["transfers"]["downloaddir"]]
+
+        bshareddirs = [path for _name, path in config["transfers"]["shared"]]
+        bshareddirs += [path for _name, path in config["transfers"]["buddyshared"]]
+        bshareddirs.append(config["transfers"]["downloaddir"])
+
         bsharedmtimes = config["transfers"]["bsharedmtimes"]
 
         dir = str(os.path.expanduser(os.path.dirname(name)))
+        vdir = self.real2virtual(dir)
         file = str(os.path.basename(name))
-        size = os.path.getsize(name)
 
-        bshared[dir] = bshared.get(dir, [])
+        bshared[vdir] = bshared.get(vdir, [])
 
-        if file not in [i[0] for i in bshared[dir]]:
+        if file not in [i[0] for i in bshared[vdir]]:
             fileinfo = self.getFileInfo(file, name)
-            bshared[dir] = bshared[dir] + [fileinfo]
-            bsharedstreams[dir] = self.getDirStream(bshared[dir])
-            words = self.getIndexWords(dir, file, bshareddirs)
-            self.addToIndex(bwordindex, bfileindex, words, dir, fileinfo)
-            bsharedmtimes[dir] = os.path.getmtime(dir)
+            bshared[vdir] = bshared[vdir] + [fileinfo]
+            bsharedstreams[vdir] = self.getDirStream(bshared[vdir])
+            words = self.getIndexWords(vdir, file, bshareddirs)
+            self.addToIndex(bwordindex, bfileindex, words, vdir, fileinfo)
+            bsharedmtimes[vdir] = os.path.getmtime(dir)
             self.newbuddyshares = True
 
     def addToIndex(self, wordindex, fileindex, words, dir, fileinfo):
-        index = len(fileindex.keys())
+        index = len(list(fileindex.keys()))
         for i in words:
             if i not in wordindex:
                 wordindex[i] = [index]
             else:
                 wordindex[i] = wordindex[i] + [index]
-        fileindex[str(index)] = (os.path.join(dir, fileinfo[0]),)+fileinfo[1:]
+        fileindex[str(index)] = (os.path.join(dir, fileinfo[0]),) + fileinfo[1:]
